local addonName, TF = ...

local TradeFill = LibStub("AceAddon-3.0"):GetAddon(addonName)
local Loc = LibStub("AceLocale-3.0"):GetLocale(addonName)

function TradeFill:InitializeHistory()
    TF.totalMoney = {
        player = 0,
        target = 0,
    }

    TF.totalHistory = {
        player = {},
        target = {},
    }

    TF.options.args.logbook.args["0"] = {
        type = "group",
        name = Loc["TOTAL"],
        desc = Loc["TOTAL_DESC"],
        order = 0,
        args = {
            all = {
                type = "execute",
                name = Loc["BUTTON_DELETE_ALL"],
                desc = Loc["BUTTON_DELETE_ALL_DESC"],
                func = function() TradeFill:DeleteHistoryAll() end,
                confirm = true,
                order = 3
            },
        },
    }

    for timeStamp, _ in pairs(TradeFill.history.profile) do
        local playerName = TradeFill.history.profile[timeStamp].player
        local targetName = TradeFill.history.profile[timeStamp].target

        TF.options.args.logbook.args[timeStamp] = {
            type = "group",
            name = date("%d %B, |cFFFFFFFF%H:%M|r", timeStamp),
            desc = date("%d-%m-%Y\n\n", timeStamp) .. targetName,
            order = -tonumber(timeStamp),
            args = {
                day = {
                    type = "execute",
                    name = Loc["BUTTON_DELETE_DAY"],
                    desc = Loc["BUTTON_DELETE_DAY_DESC"],
                    func = function() TradeFill:DeleteHistoryDay(timeStamp) end,
                    width = 0.5,
                    order = 3
                },
            },
        }

        if string.find(targetName, "-") then
            local name = string.match(targetName, "^([^%-]+)")
            targetName = name .. "|r |cFFFFFFFF(*)|r"
        end

        TF.options.args.logbook.args[timeStamp].args.player = {
            type = "group",
            name = playerName,
            desc = "desc",
            order = 1,
            dialogInline = true,
            args = {
                player = {
                    type = "description",
                    name = " ",
                    order = 1,
                }
            },
        }

        TF.options.args.logbook.args[timeStamp].args.target = {
            type = "group",
            name = targetName,
            desc = "desc",
            order = 2,
            dialogInline = true,
            args = {
                target = {
                    type = "description",
                    name = " ",
                    order = 2,
                }
            },
        }

        TradeFill:GetHistory(timeStamp, "player",  "tradePlayer")
        TradeFill:GetHistory(timeStamp, "target",  "tradeTarget")
    end

    TradeFill:GetHistoryTotal()
end

function TradeFill:GetHistoryTotal()
    TF.options.args.logbook.args["0"].args.player = {
        type = "group",
        name = Loc["TOTAL_GIVEN"],
        desc = "desc",
        order = 1,
        dialogInline = true,
        args = {
            money = {
                type = "description",
                name = "",
                order = 0,
            },
            separatorA = {
                type = "description",
                name = " ",
                order = 1,
            },
        },
    }

    TradeFill:SetPlayerMoney()

    TF.options.args.logbook.args["0"].args.target = {
        type = "group",
        name = Loc["TOTAL_RECEIVED"],
        desc = "desc",
        order = 2,
        dialogInline = true,
        args = {
            money = {
                type = "description",
                name = "",
                order = 0,
            },
            separatorA = {
                type = "description",
                name = " ",
                order = 1,
            },
        },
    }

    TradeFill:SetTargetMoney()

    for keyPlayer, trade in ipairs(TF.totalHistory["player"]) do
        TF.options.args.logbook.args["0"].args.player.args[tostring(keyPlayer)] = {
            type = "description",
            fontSize = "small",
            name = "|T" .. GetItemIcon(trade.id) .. ":28[:28[:0:6|t " .. trade.total .. " x " .. TradeFill:GetItemColor(trade.quality, trade.name),
            order = keyPlayer,
        }
    end

    for keyTarget, trade in ipairs(TF.totalHistory["target"]) do
        TF.options.args.logbook.args["0"].args.target.args[tostring(keyTarget)] = {
            type = "description",
            fontSize = "small",
            name = "|T" .. GetItemIcon(trade.id) .. ":28[:28[:0:6|t " .. trade.total .. " x " .. TradeFill:GetItemColor(trade.quality, trade.name),
            order = keyTarget,
        }
    end
end

function TradeFill:SetHistoryTotal(item, unit)
    local itemFound = false
    for _, object in ipairs(TF.totalHistory[unit]) do
        if object.name == item.name then
            object.total = object.total + item.quantity
            itemFound = true
            break
        end
    end

    if not itemFound then
        table.insert(TF.totalHistory[unit], {
            ["id"] = item.id,
            ["name"] = item.name,
            ["texture"] = item.texture,
            ["quantity"] = item.quantity,
            ["quality"] = item.quality,
            ["total"] = item.quantity
        })
    end
end

function TradeFill:GetHistory(timeStamp, unit, table)
    for tradedSlot, trade in pairs(TradeFill.history.profile[timeStamp][table]) do
        if tradedSlot == "money" and trade ~= "" then
            TF.totalMoney[unit] = TF.totalMoney[unit] + trade
            TF.options.args.logbook.args[timeStamp].args[unit].args["-1"]= {
                type = "description",
                fontSize = "medium",
                name = GetMoneyString(trade, 1000),
                order = 0,
            }
        end

        if type(tonumber(tradedSlot)) == "number" then

            if tonumber(tradedSlot) <= MAX_TRADABLE_ITEMS then
                TradeFill:SetHistoryTotal(trade, unit)
                TF.options.args.logbook.args[timeStamp].args[unit].args[tradedSlot]= {
                    type = "description",
                    fontSize = "small",
                    name = "|T" .. GetItemIcon(trade.id) .. ":28[:28[:0:6|t " .. trade.quantity .. " x " .. TradeFill:GetItemColor(trade.quality, trade.name),
                    order = tonumber(tradedSlot + 1),
                }
            else

                TF.options.args.logbook.args[timeStamp].args[unit].args[tradedSlot]= {
                    type = "header",
                    name = Loc["NOT_TRADED"],
                    order = tonumber(tradedSlot + 1),
                }

                TF.options.args.logbook.args[timeStamp].args[unit].args[tostring(tonumber(tradedSlot + 1))]= {
                    type = "description",
                    fontSize = "small",
                    name = "|T" .. GetItemIcon(trade.id) .. ":28[:28[:0:6|t " .. TradeFill:GetItemColor(trade.quality, trade.name),
                    order = tonumber(tradedSlot + 2),
                }

                if trade.enchantment ~= "" then
                    TF.options.args.logbook.args[timeStamp].args[unit].args[tostring(tonumber(tradedSlot + 2))]= {
                        type = "description",
                        fontSize = "medium",
                        name = "|cFF00FF00" .. trade.enchantment .. "|r",
                        order = tonumber(tradedSlot + 3),
                    }
                end
            end
        end
    end
end

function TradeFill:setTradeHistory()
    if TF.tradeHistory and TF.tradeHistory.time and TF.tradeHistory.time ~= "" then
        local trade = self.history.profile[TF.tradeHistory.time]

        trade.player = TF.tradeHistory.player.name
        trade.tradePlayer.money = TF.tradeHistory.player.money

        for i = 1, MAX_TRADABLE_ITEMS do
            if TF.tradeHistory.player[i] then
                for key, value in ipairs(TF.tradeHistory.player) do
                    local player = trade.tradePlayer[tostring(key)]

                    player.id = value.id
                    player.name = value.name
                    player.texture = value.texture
                    player.quantity = value.quantity
                    player.quality = value.quality
                    player.enchantment = value.enchantment
                end
            end
        end

        do
            if TF.tradeHistory.player[TRADE_ENCHANT_SLOT] then
                local player = trade.tradePlayer[tostring(TRADE_ENCHANT_SLOT)]

                player.id = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].id
                player.name = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].name
                player.texture = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].texture
                player.quantity = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].quantity
                player.quality = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].quality
                player.enchantment = TF.tradeHistory.player[TRADE_ENCHANT_SLOT].enchantment
            end
        end

        trade.target = TF.tradeHistory.target.name
        trade.tradeTarget.money = TF.tradeHistory.target.money

        for i = 1, MAX_TRADABLE_ITEMS do
            if TF.tradeHistory.target[i] then
                for key, value in ipairs(TF.tradeHistory.target) do
                    local target = trade.tradeTarget[tostring(key)]

                    target.id = value.id
                    target.name = value.name
                    target.texture = value.texture
                    target.quantity = value.quantity
                    target.quality = value.quality
                    target.enchantment = value.enchantment
                end
            end
        end

        do
            if TF.tradeHistory.target[TRADE_ENCHANT_SLOT] then
                local target = trade.tradeTarget[tostring(TRADE_ENCHANT_SLOT)]

                target.id = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].id
                target.name = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].name
                target.texture = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].texture
                target.quantity = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].quantity
                target.quality = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].quality
                target.enchantment = TF.tradeHistory.target[TRADE_ENCHANT_SLOT].enchantment
            end
        end
    end
    TradeFill:InitializeHistory()
end

function TradeFill:getTradeHistory()
    TF.tradeHistory = {
        time = "",
        player = {},
        target = {}
    }

    local player = TF.tradeHistory.player
    local target = TF.tradeHistory.target

    player.money = GetPlayerTradeMoney()
    target.money = GetTargetTradeMoney()

    for i = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(i) ~= nil then
            local name, texture, quantity, quality, _, _ = GetTradePlayerItemInfo(i)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradePlayerItemLink(i))

            if name ~= nil then
                player[i] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality,
                    enchantment = nil,
                }
            end
        end

        if GetTradeTargetItemLink(i) ~= nil then
            local name, texture, quantity, quality, _, _ = GetTradeTargetItemInfo(i)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradeTargetItemLink(i))

            if name ~= nil then
                target[i] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality,
                    enchantment = nil,
                }
            end
        end
    end

    do
        if GetTradePlayerItemLink(TRADE_ENCHANT_SLOT) ~= nil then
            local name, texture, quantity, quality, enchantment, _  = GetTradePlayerItemInfo(TRADE_ENCHANT_SLOT);
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradePlayerItemLink(TRADE_ENCHANT_SLOT))

            local enchant = enchantment or ""

            if name ~= nil then
                player[TRADE_ENCHANT_SLOT] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality,
                    enchantment = enchant,
                }
            end
        end

        if GetTradeTargetItemLink(TRADE_ENCHANT_SLOT) ~= nil then
            local name, texture, quantity, quality, _, enchantment = GetTradeTargetItemInfo(TRADE_ENCHANT_SLOT);
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradeTargetItemLink(TRADE_ENCHANT_SLOT))

            local enchant = enchantment or ""

            if name ~= nil then
                target[TRADE_ENCHANT_SLOT] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality,
                    enchantment = enchant,
                }
            end
        end
    end

    if next(player) or next(target) or player.money > 0 or target.money > 0 then
        TF.tradeHistory.time = tostring(GetServerTime())
        player.name = TradeFill:setUnitName("player")
        target.name = TradeFill:setUnitName("npc")
    end
end

function TradeFill:DeleteHistoryDay(timeStamp)
    for tradedSlot, trade in pairs(TradeFill.history.profile[timeStamp]["tradePlayer"]) do
        local moneyPlayer = TradeFill.history.profile[timeStamp].tradePlayer.money

        if tradedSlot == "money" and trade ~= "" then
            if moneyPlayer > 0 then
                TF.totalMoney.player = TF.totalMoney.player - moneyPlayer
                TradeFill:SetPlayerMoney()
            end
        end

        if type(tonumber(tradedSlot)) == "number" then
            for keyPlayer, object in ipairs(TF.totalHistory["player"]) do
                if object.name == trade.name then

                    if object.total - trade.quantity == 0 then
                        table.remove(TF.totalHistory["player"], keyPlayer)
                    else
                        object.total = object.total - trade.quantity
                    end
                end
            end
            TF.options.args.logbook.args["0"].args.player.args = {}
            TradeFill:GetHistoryTotal()
        end
    end

    for tradedSlot, trade in pairs(TradeFill.history.profile[timeStamp]["tradeTarget"]) do
        local moneyTarget = TradeFill.history.profile[timeStamp].tradeTarget.money

        if tradedSlot == "money" and trade ~= "" then
            if moneyTarget > 0 then
                TF.totalMoney.target = TF.totalMoney.target - moneyTarget
                TradeFill:SetTargetMoney()
            end
        end

        if type(tonumber(tradedSlot)) == "number" then
            for keyTarget, object in ipairs(TF.totalHistory["target"]) do
                if object.name == trade.name then

                    if object.total - trade.quantity == 0 then
                        table.remove(TF.totalHistory["target"], keyTarget)
                    else
                        object.total = object.total - trade.quantity
                    end
                end
            end
            TF.options.args.logbook.args["0"].args.target.args = {}
            TradeFill:GetHistoryTotal()
        end
    end

    if TF.options.args.logbook.args[timeStamp] then
        TF.options.args.logbook.args[timeStamp] = nil
        TradeFill.history.profile[timeStamp] = nil

        DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["REMOVED_TRADE"] .. date("%d %B, |cFFFFFFFF%H:%M|r", timeStamp))
    end
end

function TradeFill:DeleteHistoryAll()
    for timeStamp, _ in pairs(TradeFill.history.profile) do
        TF.options.args.logbook.args[timeStamp] = nil
    end

    TF.totalMoney.player = 0
    TF.totalMoney.target = 0

    TF.totalHistory.player = {}
    TF.totalHistory.target = {}

    TF.options.args.logbook.args["0"].args = {}
    TF.options.args.logbook.args["0"].args = {}

    self.history:ResetProfile()

    TradeFill:GetHistoryTotal()

    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["REMOVED_ALL"])
end

function TradeFill:SetPlayerMoney()
    TF.options.args.logbook.args["0"].args.player.args.money.name = GetMoneyString(TF.totalMoney["player"], 1000)
end

function TradeFill:SetTargetMoney()
    TF.options.args.logbook.args["0"].args.target.args.money.name = GetMoneyString(TF.totalMoney["target"], 1000)
end

function TradeFill:setUnitName(unit)
    local _, classFile, _ = UnitClass(unit)
    local _, _, _, argbHex = GetClassColor(classFile)

    if classFile == "SHAMAN" then
        argbHex = "FF0070DE"
    end

    return "|c".. argbHex .. GetUnitName(unit, true) .. "|r"
end